/**
 * Console for testing/debugging over serial port
 */
package com.sea.gbgs5;

import com.cinterion.io.InPort;
import com.cinterion.io.InPortListener;
import com.sea.log.Logger;
import java.io.IOException;
import java.util.Vector;

/**
 *
 * @author vorisek
 */
public class ConsoleReporter extends Thread implements InPortListener {
    
// --------------------------------------------------
// Public Properties
// --------------------------------------------------
// --------------------------------------------------
// Private Constants
// --------------------------------------------------
// --------------------------------------------------
// Private Properties
// --------------------------------------------------

private static ConsoleReporter instance;
private boolean bRunning;
//private SyncAtCommand atc;
public static final int iLogLevel = Logger.INFO;

private InPort inp;
private int lastPortValue;

private boolean bAuto;
private int iAutoMode;

// --------------------------------------------------
// Class Code
// --------------------------------------------------
public ConsoleReporter() {
  super();
  bAuto = false;
  iAutoMode = 0;
}

/**
 * getInstance
 */
public static ConsoleReporter getInstance() {
  if (instance == null) {
    instance = new ConsoleReporter();
  }
  return instance;
}

/**
 * startRunning
 */
public void startRunning() {
  bRunning = true;
  this.start();
}

/**
 * stopRunning
 */
public void stopRunning() {
  bRunning = false;
  this.interrupt();
}

/**
 * Debug printing
 */
protected static void log(String line) {
  log(line, Logger.INFO);
}

/**
 * Debug printing
 */
protected static void log(String line, int level) {
  if (iLogLevel >= level) {
    Logger.log("@Console", line);
  }
}

/**
 * run
 */
public void run() {

  Vector pins = new Vector();
  pins.addElement("GPIO8"); // X2
  pins.addElement("GPIO7"); // X1
  try {
    inp = new InPort(pins);
    lastPortValue = inp.getValue();
    inp.addListener(this);
  } catch (IOException ex) {
    ex.printStackTrace();
  }

  //log("Thread started", Logger.ALWAYS);
  while (bRunning) {
    try {

      try {
        sleep(1000);
      } catch (InterruptedException ignored) {
      }

      if (bAuto) {
        printReport();
      }

      //} catch (InterruptedException ignored) {
    } catch (Exception e) {
      // An exception occured. Go the interrupted step again.
      log("[ EXCEPTION ] - '" + e.toString() + "'", Logger.ERROR);
      e.printStackTrace();

    }

  }

  try {
    inp.release();
  } catch (IOException ignored) {
  }

  //log("Thread finished", Logger.ALWAYS);
}

void reportOnceAndPause() {
  bAuto = false;
  printReport();
}

void resume(int mode) {
  iAutoMode = mode;
  bAuto = true;
  this.interrupt();
}

public void portValueChanged(int portValue) {
  //log("inPortValueChanged() - value = " + Integer.toHexString(portValue) + " lastValue = " + lastPortValue);
  lastPortValue = portValue;
  this.interrupt();
}

private void printReport() {
  if (Main.SERIAL_NUMBER.startsWith("GD1")) {
    // GSM-KLIC
    log("STATE - X1=" + ((lastPortValue >> 1) & 0x01) + ";X2=" + (lastPortValue & 0x01) + ";Y3=" + Slave.getInstance().getOut() + ";A4="
      + "?" + " ADC" + ";");
  } else {
    log("STATE - A1=?" + " Pa" + ";X2=" + lastPortValue + ";Y3=" + Slave.getInstance().getOut() + ";A4="
      + "?" + " ADC" + ";");
  }
}

}
